﻿/*
 * Author:       Michael Meijer
 * Last changed: November 18, 2012
 */

using System;
using System.Collections.Generic;
using Microsoft.ComplexEventProcessing.Linq;

namespace Msdn.StreamInsight
{
    /// <summary>
    /// A utility class for the stream operators and event sources / sinks
    /// </summary>
    public static partial class Utility
    {
        /// <summary>
        /// Extension method wrapping the approximate count operator for a source stream.
        /// </summary>
        /// <param name="source">The event source stream.</param>
        /// <param name="n">Window size for the operator.</param>
        /// <param name="epsilon">Relative error upper bound of the operator's estimates.</param>
        /// <returns>The event sink stream.</returns>
        public static IQStreamable<long> ApproximateCount(this IQStreamable<bool> source, long n, double epsilon)
        {
            return source.Scan(() => new ApproximateCountUDSO(n, epsilon));
        }

        /// <summary>
        /// Seeded uniform random number generator.
        /// </summary>
        private static Random random = new Random((int)DateTime.Now.Ticks);

        /// <summary>
        /// Generates the paylods. Uses the yield-return iterator syntax.
        /// 
        /// Note: this method is not thread-safe.
        /// </summary>
        /// <returns>Payloads of either 0/False or 1/True.</returns>
        public static IEnumerable<bool> EnumeratePayloads()
        {
            // ad infinitum
            while (true)
            {
                bool payload = random.NextDouble() >= 0.5;

                yield return payload;
            }
        }
    }
}
